/* SmoothWall setup program.
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Lawrence Manning, 2001
 *
 * filename: main.c
 * Contains main entry point, and misc functions. */

#include "setup.h"

FILE *flog = NULL;
char *log;

char **ctr = NULL;

int automode = 0;

extern char *english_tr[];
extern char *afrikaans_tr[];
extern char *german_tr[];
extern char *italian_tr[];
extern char *french_tr[];
extern char *norwegian_tr[];
extern char *romanian_tr[];
extern char *danish_tr[];
extern char *dutch_tr[];
extern char *spanish_tr[];
extern char *finnish_tr[];
extern char *swedish_tr[];
extern char *brazilian_tr[];
extern char *hungarian_tr[];

int main(int argc, char *argv[])
{
	char *shortlangnames[] = { "za", "br", "dk", "de", "en", "es", "fr", "it",
		"hu", "nl", "no", "ro", "fi", "se", NULL };
	char **langtrs[] = { afrikaans_tr, brazilian_tr, danish_tr, german_tr, english_tr,
		spanish_tr, french_tr, italian_tr, hungarian_tr, dutch_tr, norwegian_tr,
		romanian_tr, finnish_tr, swedish_tr, NULL };
	int choice;
	char *sections[10]; /* need to fill this out AFTER knowning lang */
	int rc;
	struct keyvalue *kv;
	char selectedshortlang[STRING_SIZE] = "en";
	int langcounter;
	int autook = 0;
	
	/* Log file/terminal stuff. */
	if (argc >= 2)
		log = argv[1];	
	else
		log = strdup("/root/setup.log");

	if (!(flog = fopen(log, "w+")))
	{
		printf("Couldn't open log terminal\n");
		return 1;
	}
	
	if (argc >= 3)
		automode = 1;
	
	fprintf(flog, "Setup program started.\n");
		
	kv = initkeyvalues();
	if (!(readkeyvalues(kv, CONFIG_ROOT "main/settings")))
	{
		printf("SmoothWall is not properly installed.\n");
		return 1;
	}
	findkey(kv, "LANGUAGE", selectedshortlang);
	
	for (langcounter = 0; langtrs[langcounter]; langcounter++)
	{
		if (strcmp(selectedshortlang, shortlangnames[langcounter]) == 0)
		{
			ctr = langtrs[langcounter];
			break;
		}
	}

	if (!ctr)
	{
		printf("Selected language not supported by setup.\n");
		return 1;
	}

	sections[0] = ctr[TR_KEYBOARD_MAPPING];
	sections[1] = ctr[TR_TIMEZONE];
	sections[2] = ctr[TR_HOSTNAME];
	sections[3] = ctr[TR_ISDN_CONFIGURATION];
	sections[4] = ctr[TR_USBADSL_CONFIGURATION];
	sections[5] = ctr[TR_NETWORKING];	
	sections[6] = ctr[TR_DHCP_SERVER_CONFIGURATION],
	sections[7] = ctr[TR_ROOT_PASSWORD];
	sections[8] = ctr[TR_SETUP_PASSWORD];
	sections[9] = ctr[TR_ADMIN_PASSWORD];
	sections[10] = NULL;
	
	newtInit();
	newtCls();

	newtDrawRootText(7, 0, "SmoothWall - Lawrence Manning, Richard Morrell www.smoothwall.org");
	newtPushHelpLine(ctr[TR_HELPLINE]);		

	if (automode == 0)
	{
		choice = 0;
		for (;;)
		{
			rc = newtWinMenu(ctr[TR_SECTION_MENU],
				ctr[TR_SELECT_THE_ITEM], 50, 5, 5, 10,
				sections, &choice, ctr[TR_OK], ctr[TR_QUIT], NULL);
			
			if (rc == 2)
				break;
			
			switch (choice)
			{
				case 0:
					handlekeymap();
					break;
				
				case 1:
					handletimezone();
					break;
					
				case 2:
					handlehostname();
					break;
				
				case 3:
					handleisdn();
					break;

				case 4:
					handleusbadsl();
					break;
				
				case 5:
					handlenetworking();
					break;
					
				case 6:
					handledhcp();
					break;
									
				case 7:
					handlerootpassword();
					break;

				case 8:
					handlesetuppassword();
					break;
					
				case 9:
					handleadminpassword();
					break;
		
				default:
					break;
			}
		}
	}
	else
	{
		if (!(handlekeymap()))
			goto EXIT;
		if (!(handletimezone()))
			goto EXIT;	
		if (!(handlehostname()))
			goto EXIT;
		if (!(handleisdn()))
			goto EXIT;
		if (!(handleusbadsl()))
			goto EXIT;
		if (!(handlenetworking()))
			goto EXIT;
		if (!(handledhcp()))
			goto EXIT;
		if (!(handlerootpassword()))
			goto EXIT;
		if (!(handlesetuppassword()))
			goto EXIT;
		if (!(handleadminpassword()))
			goto EXIT;

		autook = 1;
	}

EXIT:	
	if (automode != 0)
	{
		if (autook)
			newtWinMessage(TITLE, ctr[TR_OK], ctr[TR_SETUP_FINISHED]);
		else
			newtWinMessage(ctr[TR_WARNING], ctr[TR_OK], ctr[TR_SETUP_NOT_COMPLETE]);
	}
	
	fprintf(flog, "Setup program ended.\n");
	fflush(flog);
	fclose(flog);
		
	newtFinished();

	return 0;
}

