/* SmoothWall setup program.
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Lawrence Manning, 2001
 *
 * filename: usbadsl.c
 * USB ADSL things. */
 
#include "setup.h"
 
extern FILE *flog;
extern char *log;

extern char **ctr;

extern int automode;

struct controller
{
	char *name;
	char *module;
};

struct controller controllers[] = {
	{ "", "" },
	{ "UHCI", "usb-uhci" },
	{ "OHCI", "usb-ohci" },
	{ NULL, NULL }
};

static int usbadslenabledpressed(void);
static int usbadsldisabledpressed(void);
static void handleusbcontroller(void);
static int probeusbcontroller(char *controller);

int handleusbadsl(void)
{
	struct keyvalue *kv;
	int rc;
	char controllername[STRING_SIZE] = "";
	char temp[STRING_SIZE];
	char message[1000];
	int c;
	char *sections[] = { ctr[TR_USB_CONTROLLER], NULL };
	int choice;
	char enableddisabled[STRING_SIZE];
	FILE *f;
	
	if ((f = fopen(CONFIG_ROOT "red/active", "r")))
	{
		fclose(f);
		errorbox(ctr[TR_RED_IN_USE]);
		return 1;
	}
	
	/* usbadsl-cleanup is a small script to bring down ippp0 and kill ipppd
	 * and removes the USB modules. */
	mysystem("/etc/ppp/usbadsl-cleanup");
	mysystem("/sbin/modprobe usbcore");

	choice = 0;	
	for (;;)
	{
		kv = initkeyvalues();
		if (!(readkeyvalues(kv, CONFIG_ROOT "usbadsl/settings")))
		{
			freekeyvalues(kv);
			errorbox(ctr[TR_UNABLE_TO_OPEN_SETTINGS_FILE]);
			return 0;
		}

		strcpy(enableddisabled, ctr[TR_DISABLED]);
		findkey(kv, "ENABLED", temp);
		if (strcmp(temp, "on") == 0)
			strcpy(enableddisabled, ctr[TR_ENABLED]);
						
		findkey(kv, "TYPE", temp);
		c = 0;
		while (controllers[c].module)
		{
			if (strcmp(controllers[c].module, temp) == 0)
			{
				strcpy(controllername, controllers[c].name);
				break;
			}
			c++;
		}
		if (!strlen(controllername))
			strcpy(controllername, ctr[TR_UNSET]);		

		sprintf(message, ctr[TR_USBADSL_STATUS], enableddisabled, controllername);
		
		freekeyvalues(kv);
		
		rc = newtWinMenu(ctr[TR_USBADSL_CONFIGURATION], message, 50, 5, 5, 6,
			sections, &choice, ctr[TR_OK], ctr[TR_ENABLE_USBADSL],
			ctr[TR_DISABLE_USBADSL], NULL);
		
		if (rc == 1 || rc == 0)
		{
			switch (choice)
			{
				case 0:
					handleusbcontroller();
					break;

				default:
					break;
			}
		}
		else if (rc == 2)
		{
			if (!usbadslenabledpressed())
				break;
		}
		else
		{
			if (!(usbadsldisabledpressed()))
				break;
		}	
	}
	
	return 1;
}

/* Returns 0 if main USB ADSL setup loop should exit. */
static int usbadsldisabledpressed(void)
{
	struct keyvalue *kv = initkeyvalues();

	if (!(readkeyvalues(kv, CONFIG_ROOT "usbadsl/settings")))
	{
		freekeyvalues(kv);
		errorbox(ctr[TR_UNABLE_TO_OPEN_SETTINGS_FILE]);
		return 0;
	}

	replacekeyvalue(kv, "ENABLED", "off");
	writekeyvalues(kv, CONFIG_ROOT "usbadsl/settings");

	freekeyvalues(kv);
	
	return 0;
}

/* Returns 0 if main USB ADSL setup loop should exit. */
static int usbadslenabledpressed(void)
{
	struct keyvalue *kv = initkeyvalues();
	char controller[STRING_SIZE] = "";
	char commandstring[STRING_SIZE];
	int result = 0;

	if (!(readkeyvalues(kv, CONFIG_ROOT "usbadsl/settings")))
	{
		freekeyvalues(kv);
		errorbox(ctr[TR_UNABLE_TO_OPEN_SETTINGS_FILE]);
		return 0;
	}

	findkey(kv, "CONTROLLER", controller);
				
	if (strlen(controller))
	{
		sprintf(commandstring, "/sbin/modprobe %s", controller);
		if (runcommandwithstatus(commandstring, ctr[TR_INITIALISING_USBADSL]) != 0)
		{
			errorbox(ctr[TR_UNABLE_TO_INITIALISE_USBADSL]);
			replacekeyvalue(kv, "ENABLED", "off");
			result = 1;		
		}
		else
		{
			if (automode == 0)
				runcommandwithstatus("/etc/rc.d/rc.usbadsl",
					ctr[TR_STARTING_UP_USBADSL]);
			replacekeyvalue(kv, "ENABLED", "on");
		}
	}
	else
	{
		errorbox(ctr[TR_USBADSL_NOT_SETUP]);
		replacekeyvalue(kv, "ENABLED", "off");
		result = 1;
	}
	writekeyvalues(kv, CONFIG_ROOT "usbadsl/settings");

	freekeyvalues(kv);
	
	return result;
}

static void handleusbcontroller(void)
{
	char **selection;
	int c;
	int rc;
	int choice;
	struct keyvalue *kv = initkeyvalues();
	char temp[STRING_SIZE];
	char controller[STRING_SIZE];
	char message[STRING_SIZE];
	char commandstring[STRING_SIZE];
	int done = 0;
	
	if (!(readkeyvalues(kv, CONFIG_ROOT "usbadsl/settings")))
	{
		freekeyvalues(kv);
		errorbox(ctr[TR_UNABLE_TO_OPEN_SETTINGS_FILE]);
		return;
	}
	
	strcpy(temp, ""); findkey(kv, "CONTROLLER", temp);

	/* Count controllers. */
	c = 0;
	while (controllers[c].name) c++;
	selection = malloc((c + 1) * sizeof(char *));

	/* Fill out section. */
	c = 0;
	selection[c] = ctr[TR_AUTODETECT];
	c++;
	while (controllers[c].name)
	{
		selection[c] = controllers[c].name;
		c++;
	}
	selection[c] = NULL;

	/* Determine inital value for choice. */
	c = 0; choice = 0;
	while (controllers[c].name)
	{
		if (strcmp(controllers[c].module, temp))
		{
			choice = c;
			break;
		}
		c++;
	}
	
	while (!done)
	{
		rc = newtWinMenu(ctr[TR_USB_CONTROLLER_SELECTION], 
			ctr[TR_CHOOSE_THE_USB_CONTROLLER_INSTALLED],
			50, 5, 5, 10, selection, &choice, ctr[TR_OK], ctr[TR_CANCEL], NULL);
	
		if (rc == 2)
			done = 1;
		else
		{	
			if (choice == 0)
				probeusbcontroller(controller);
			else
			{
				sprintf(message, ctr[TR_CHECKING_FOR], controllers[choice].name);
				sprintf(commandstring, "/sbin/modprobe %s", controllers[choice].module);
				if (runcommandwithstatus(commandstring, message) == 0)
					strcpy(controller, controllers[choice].module);
				else
				{
					errorbox(ctr[TR_USB_CONTROLLER_NOT_DETECTED]);
					strcpy(controller, "");
				}
				mysystem("/etc/ppp/usbadsl-cleanup");
			}
			if (strlen(controller))
			{
				replacekeyvalue(kv, "CONTROLLER", controller);
				writekeyvalues(kv, CONFIG_ROOT "usbadsl/settings");
				done = 1;
			}
		}
	}

	free(selection);	
	freekeyvalues(kv);
}

static int probeusbcontroller(char *controller)
{
	int c;
	char message[STRING_SIZE];
	char commandstring[STRING_SIZE];
	struct keyvalue *kv = initkeyvalues();
	int result = -1;
	
	if (!(readkeyvalues(kv, CONFIG_ROOT "usbadsl/settings")))
	{
		freekeyvalues(kv);
		errorbox(ctr[TR_UNABLE_TO_OPEN_SETTINGS_FILE]);
		return  -1;
	}
		
	c = 1;
	while (controllers[c].name)
	{
		sprintf(message, ctr[TR_CHECKING_FOR], controllers[c].name);
		sprintf(commandstring, "/sbin/modprobe %s", controllers[c].module);
		
		if (runcommandwithstatus(commandstring, message) == 0)
		{
			mysystem("/etc/ppp/usbadsl-cleanup");
			sprintf(message, ctr[TR_DETECTED], controllers[c].name);
			newtWinMessage(TITLE, ctr[TR_OK], message);
			strcpy(controller, controllers[c].module);
			goto EXIT;
		}
		c++;
	}

	errorbox(ctr[TR_UNABLE_TO_FIND_A_USB_CONTROLLER]);
	
EXIT:
	freekeyvalues(kv);
	
	return result;
}
