/* SmoothWall helper program - restartdhcp
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Daniel Goscomb, 2001
 * 
 * Modifications and improvements by Lawrence Manning.
 *
 * 10/04/01 Aslak added protocol support
 *
 * filename: setportfwd.c
 * This program reads the list of ports to forward and setups ipchains
 * and rules in ipmasqadm to enable them. */

#include "setuid.h"

int main(void)
{
	FILE *ipfile = NULL, *ifacefile = NULL;
	FILE *fwdfile = NULL;
	int count;
	char iface[STRING_SIZE];
	char locip[STRING_SIZE];
	char *protocol;
	char *locport;
	char *remip;
	char *remport;
	char *enabled;
	char s[STRING_SIZE];
	char *result;
	char command[STRING_SIZE];

	if (!(initsetuid()))
		exit(1);
	
	if (!(ipfile = fopen("/var/smoothwall/red/local-ipaddress", "r")))
	{
		fprintf(stderr, "Couldn't open local ip file");
		goto EXIT;
	}
	fgets(locip, STRING_SIZE, ipfile);
	if (locip[strlen(locip) - 1] == '\n')
		locip[strlen(locip) - 1] = '\0';
 	fclose (ipfile);
 	ipfile = NULL; 	
 	if (strspn(locip, IP_NUMBERS) != strlen(locip))
 	{
 		fprintf(stderr, "Bad local IP: %s\n", locip);
 		goto EXIT;
 	}
 	
	if (!(ifacefile = fopen("/var/smoothwall/red/iface", "r")))
	{
		fprintf(stderr, "Couldn't open iface file");
		goto EXIT;
	}
	fgets(iface, STRING_SIZE, ifacefile);
	if (iface[strlen(iface) - 1] == '\n')
		iface[strlen(iface) - 1] = '\0';	
 	fclose (ifacefile);
 	ifacefile = NULL;
 	if (strspn(iface, LETTERS_NUMBERS) != strlen(iface))
 	{
 		fprintf(stderr, "Bad iface: %s\n", iface);
 		goto EXIT;
 	}
 	
 	if (!(fwdfile = fopen("/var/smoothwall/portfw/config", "r")))
	{
		fprintf(stderr, "Couldn't open portfw settings file\n");
		goto EXIT;
	}

	system("/usr/sbin/ipmasqadm portfw -f");

	while (fgets(s, STRING_SIZE, fwdfile) != NULL)
	{
		if (s[strlen(s) - 1] == '\n')
		        s[strlen(s) - 1] = '\0';		        	
		result = strtok(s, ",");

		count = 0;
		protocol = NULL;
		locport = NULL;
		remip = NULL;
		remport = NULL;
		enabled = NULL;
		while (result)
		{
			if (count == 0)
				protocol = result;
			else if (count == 1)
				locport = result;
			else if (count == 2)
				remip = result;
			else if (count == 3)
				remport = result;
			else
				enabled = result;
			count++;
			result = strtok(NULL, ",");
		}
		
		if (!(protocol && locport && remip && remport && enabled))
			break;
		
		if (strspn(protocol, LETTERS) != strlen(protocol))
		{
			fprintf(stderr, "Bad protocol: %s\n", protocol);
			goto EXIT;
		}
		if (strspn(locport, NUMBERS) != strlen(locport))
		{
			fprintf(stderr, "Bad loal port: %s\n", locport);
			goto EXIT;
		}
		if (strspn(remip, IP_NUMBERS) != strlen(remip))
		{
			fprintf(stderr, "Bad remote IP: %s\n", remip);
			goto EXIT;
		}
		if (strspn(remport, NUMBERS) != strlen(remport))
		{
			fprintf(stderr, "Bad remote port: %s\n", remport);
			goto EXIT;
		}
		if (strcmp(enabled, "on") == 0)
		{
			memset(command, 0, STRING_SIZE);
			snprintf(command, STRING_SIZE - 1, "/usr/sbin/ipmasqadm portfw -a -P %s -L %s %s -R %s %s", protocol, locip, locport, remip, remport);
			system(command);
		}
	}
		
EXIT:
	if (ipfile)
		fclose(ipfile);
	if (ifacefile)
		fclose(ifacefile);
	if (fwdfile)
		fclose(fwdfile);
	
	return 0;
}
